// xTextEdit.cpp : Definiert den Einsprungpunkt fr die DLL-Anwendung.
//
#include "stdafx.h"
#include "VerInfo.h"

#define StringCchCopy(a, b, c) strncpy((a), (c), (b))
#define StringCchLength(a, b, c) *(c) = strlen((a)), S_OK
#define StringCchPrintf _snprintf

//Textedit-2.52
#include <fstream>
#include "regexp.h"
#include "regmagic.h"

HINSTANCE hInstance;
HWND messageHandler;

bool userConfirmation = GetRCBoolDef("xTextEditConfirmation", false);
bool userDebug = GetRCBoolDef("xTextEditDebug", false);

void getToken(char line[MAX_LINE_LENGTH], char temp[MAX_LINE_LENGTH], bool escape_codes = true);

void BangTextAppend(HWND hwndCaller, LPCSTR pszArgs);
void BangTextDelete(HWND hwndCaller, LPCSTR pszArgs);
void BangTextReplace(HWND hwndCaller, LPCSTR pszArgs);
void BangTextReplaceAll(HWND hwndCaller, LPCSTR pszArgs);
void BangTextSaveEvar(HWND hwndCaller, LPCSTR pszArgs);

int lsMessages[] = {
	LM_GETREVID,
	0
};

LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case LM_GETREVID:
		{
			UINT uLength;
			StringCchPrintf((char*)lParam, 64, "%s.dll: %s", V_NAME, V_VERSION);
			
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;

			lParam = NULL;
			return 0;
		}
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes);

BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);

	return TRUE;
}

int initModuleEx(HWND hParent, HINSTANCE hInstance, const char *lsPath)
{
	WNDCLASSEX wc;

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = 0;
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xTextEditMessageHandler";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	messageHandler = CreateWindowEx(WS_EX_TOOLWINDOW,
		"xTextEditMessageHandler",
		0,
		WS_POPUP,
		0, 0, 0, 0, 
		0,
		0,
		hInstance,
		0);

	if (!messageHandler)
		return 1;

	SendMessage(GetLitestepWnd(),
		LM_REGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	::hInstance = hInstance;

	AddBangCommand("!xTextAppend", BangTextAppend);
	AddBangCommand("!xTextDelete", BangTextDelete);
	AddBangCommand("!xTextReplace", BangTextReplace);
	AddBangCommand("!xTextReplaceAll", BangTextReplaceAll);
	AddBangCommand("!xTextSaveEvar", BangTextSaveEvar);

	return 0;
}

void quitModule(HINSTANCE hInstance)
{
	RemoveBangCommand("!xTextAppend");
	RemoveBangCommand("!xTextDelete");
	RemoveBangCommand("!xTextReplace");
	RemoveBangCommand("!xTextReplaceAll");
	RemoveBangCommand("!xTextSaveEvar");

	SendMessage(GetLitestepWnd(),
		LM_UNREGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	DestroyWindow(messageHandler);

	UnregisterClass("xTextEditMessageHandler", hInstance);
}



//----------------------------------------------------------------
//----------------------------------------------------------------

	//=========================================================
	// getToken grabs a token and clobbers info that is parsed
	//=========================================================
	void getToken(char line[MAX_LINE_LENGTH], char temp[MAX_LINE_LENGTH], bool escape_codes /*= true*/)
	{
		int offset = 0, i;
		bool endquote = false;
		char ENCAP = '@';

		while (line[offset] == ' ')
		{
			offset++;
		}

		if (line[offset] == ENCAP)
		{
			offset++;
			endquote = true;
		}

		for (i = 0; i+offset < int(strlen(line)); ++i)
		{
			// Copy else if block to add more escape codes.
			if (escape_codes && (line[i+offset] == '\\' && int(strlen(line)) != i+offset+1))
			{
				if (line[i+offset+1] == ENCAP || line[i+offset+1] == '\\')
				{
					offset++;
					temp[i] = line[i+offset];
					continue;
				}
				// Bang escape code below.
				else if (line[i+offset+1] == '^')
				{
					offset++;
					temp[i] = '!';
					continue;
				}
				// EVar escape code below.
				else if (line[i+offset+1] == '#')
				{
					offset++;
					temp[i] = '$';
					continue;
				}
				// Comment escape code below.
				else if (line[i+offset+1] == '~')
				{
					offset++;
					temp[i] = ';';
					continue;
				}
				// Quotation mark escape code below.
				else if (line[i+offset+1] == '=')
				{
					offset++;
					temp[i] = '"';
					continue;
				}
			}

 			if (line[i+offset] == ENCAP && endquote)
	  			break;
			else if (line[i+offset] == ' ' && !endquote)
				break;
			else
				temp[i] = line[i+offset];
		}
		temp[i] = '\0';

		if (i+offset+1 > int(strlen(line)))
		{
			line[0] = '\0';
		}
		else
		{
			strcpy(line, line+i+offset+1);
		}
	}

	//=========================================================
	// Bang command handling
	//=========================================================

	void BangTextAppend(HWND hwndCaller, LPCSTR pszArgs)
	{
		char filename[MAX_LINE_LENGTH], str[MAX_LINE_LENGTH];
		char rest[MAX_LINE_LENGTH];
		strcpy(rest, pszArgs);

		getToken(rest, filename, false);
		getToken(rest, str, true);

		ofstream file;
		file.open(filename, /* ios::nocreate | */ ios::app);

		if (!file.fail())
		{
			file << str << endl;
		}

		file.close();
	}

	void BangTextDelete(HWND hwndCaller, LPCSTR pszArgs)
	{
		char filename[MAX_LINE_LENGTH], regex[MAX_LINE_LENGTH], input[MAX_LINE_LENGTH];
		char rest[MAX_LINE_LENGTH];
		regexp *r;
		strcpy(rest, pszArgs);

		getToken(rest, filename, false);
		getToken(rest, regex, true);

		r = regcomp(regex);

		ifstream file;
		file.open(filename/*, ios::nocreate*/);        //removed because dev-cpp wont compile it... feel free to recompile to get it to work

		if (!file.fail())
		{
			list<string> l;
			string s;
			while (file.getline(input, MAX_LINE_LENGTH))
			{
				if (!regexec(r, input))
				{
					s = input;
					l.push_back(s);
				}
				else
				{
					if ( userConfirmation || userDebug )
					{
						string message = "Do you really want to delete\n\n";
						message.append( input );
						message.append ( "\n\n in the file\n\n" );
						message.append ( filename );
						if ( userDebug )
						{
							message.append( "\n\nSearch String was:\n\n" );
							message.append( regex );
						}
						if ( MessageBox(NULL, message.c_str(), "User Confirmation", MB_YESNO | MB_SETFOREGROUND | MB_ICONQUESTION) == IDNO )
						{
							s = input;
							l.push_back(s);
						}
					}
				}
			}
			file.close();
			ofstream outfile;
			outfile.open(filename);
			for (list<string>::const_iterator lci = l.begin(); lci!=l.end(); lci++)
			{
				outfile << lci->data() << endl;
			}
			l.clear();
		}
	}

	void BangTextReplace(HWND hwndCaller, LPCSTR pszArgs)
	{
		char filename[MAX_LINE_LENGTH], regex[MAX_LINE_LENGTH], input[MAX_LINE_LENGTH], replace[MAX_LINE_LENGTH];
		char rest[MAX_LINE_LENGTH], buf[MAX_LINE_LENGTH];
		regexp *r;
		strcpy(rest, pszArgs);

		getToken(rest, filename, false);
		getToken(rest, regex, true);
		getToken(rest, replace, true);

		r = regcomp(regex);

		ifstream file;
		file.open(filename/*, ios::nocreate*/);        //removed because dev-cpp wont compile it... feel free to recompile to get it to work

		bool found = false;
		if (!file.fail())
		{
			
			list<string> l;
			string s;
			while (file.getline(input, MAX_LINE_LENGTH))
			{
				if ( !found && regexec(r, input) )
				{
					regsub(r, replace, buf);

					if ( !userConfirmation && !userDebug)
					{
						s = buf;
						l.push_back(buf);
						found = true;
					}
					else
					{
						string message = "Do you want to replace\n\n";
						message.append( input );
						message.append( "\nwith\n" );
						message.append ( buf );
						message.append ( "\n\n in the file\n\n" );
						message.append ( filename );
						if ( userDebug )
						{
							message.append( "\n\nSearch String was:\n" );
							message.append( regex );
							message.append( "\n\nReplace String was:\n" );
							message.append( replace );
						}
						if ( MessageBox(NULL, message.c_str(), "User Confirmation", MB_YESNO | MB_SETFOREGROUND | MB_ICONQUESTION ) == IDYES )
						{
							found = true;
							s = buf;
							l.push_back(buf);
						}
						else
						{
							found = true;
							s = input;
							l.push_back(input);
						}
					}
				}
				else
				{
					s = input;
					l.push_back(input);
				}
			}
			file.close();
			ofstream outfile;
			outfile.open(filename);
			for (list<string>::const_iterator lci = l.begin(); lci!=l.end(); lci++)
			{
				outfile << lci->data() << endl;
			}
			l.clear();
		}

		if ( !found )
		{
			string message = "The given Search String\n\n";
			message.append( regex );
			message.append ( "\n\nwasn't found in the file\n\n" );
			message.append ( filename );
			message.append ( "\n\nReplace canceled!" );
			MessageBox(NULL, message.c_str(), "User Confirmation", MB_OK | MB_SETFOREGROUND | MB_ICONEXCLAMATION);
		}
	}

	void BangTextReplaceAll(HWND hwndCaller, LPCSTR pszArgs)
	{
		char filename[MAX_LINE_LENGTH], regex[MAX_LINE_LENGTH], input[MAX_LINE_LENGTH], replace[MAX_LINE_LENGTH];
		char rest[MAX_LINE_LENGTH], buf[MAX_LINE_LENGTH];
		regexp *r;
		strcpy(rest, pszArgs);

		getToken(rest, filename, false);
		getToken(rest, regex, true);
		getToken(rest, replace, true);

		r = regcomp(regex);

		ifstream file;
		file.open(filename/*, ios::nocreate*/);        //removed because dev-cpp wont compile it... feel free to recompile to get it to work

		if (!file.fail())
		{
			list<string> l;
			string s;
			while (file.getline(input, MAX_LINE_LENGTH))
			{
				if (regexec(r, input))
				{
					s = buf;
					l.push_back(buf);
				}
				else
				{
					s = input;
					l.push_back(input);
				}
			}
			file.close();
			ofstream outfile;
			outfile.open(filename);
			for (list<string>::const_iterator lci = l.begin(); lci!=l.end(); lci++)
			{
				outfile << lci->data() << endl;
			}
			l.clear();
		}
	}

	void BangTextSaveEvar(HWND hwndCaller, LPCSTR pszArgs)
	{
		char filename[MAX_LINE_LENGTH] = {0};
		char str[MAX_LINE_LENGTH] = {0};
		char val[MAX_LINE_LENGTH] = {0};
		char rest[MAX_LINE_LENGTH] = {0};

		strcpy(rest, pszArgs);

		getToken(rest, filename, false);
		getToken(rest, str, false);
		getToken(rest, val, true);

		string file = filename;
		string EvarToSave = str;
		string result = "";
			
		int length = EvarToSave.length();
		int i = 0;
			
		while(i < length)
		{
			result.append(1, '[');
			result.append(1, tolower(EvarToSave[i]));
			result.append(1, toupper(EvarToSave[i++]));
			result.append(1, ']');
		}
			
		EvarToSave = result;

		result = val;
		if ( result.empty() )
		{
			result = "$";
			result.append( str );
			result.append( "$" );
			VarExpansion( str, result.c_str() );
		}
		else
			strcpy( str, val);

		result = str;

		BangTextReplace(NULL, ("@" + file + "@ @[		]*(" + EvarToSave + ").*@ @\\1 \"" + result + "\"@").c_str());
	}

//----------------------------------------------------------------
//----------------------------------------------------------------
